# -*- coding: utf-8 -*-
##############################################################################
#
#    Email marketing click & read tracker
#    Copyright (C) 2011 Florent Aide, <florent.aide@gmail.com>
#    Copyright (C) 2011 XCG Consulting (http://www.xcg-consulting.fr/)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from replace import insert_tracker_in_html, insert_tracker_in_text

class hooked_email_template(osv.osv):
    """overrides the base email template to hook the URL tracker
    """
    _inherit = 'email.template'

    def generate_mail(self, cursor, user, template_id,
                      record_ids, context=None):
        """since we cannot override the _generate_mailbox_item_from_template
        method we hook into the higher level generate mail and are forced to
        copy over the code in order to add our own... This is some KIND of
        inheritance :)
        """

        if context is None:
            context = {}

        template = self.browse(cursor, user, template_id, context=context)

        if not template:
            raise Exception("The requested template could not be loaded")

        # CHANGE FROM ORIGINAL METHOD
        if context.get('use_tracker', False):
            # here we need to use tracker so let's do it
            tracker_base = context.get('tracker_base')
            text = template.def_body_text
            template.def_body_text, tracks = insert_tracker_in_text(
                text, tracker_base, context['activity_id'])

            trackitem = self.pool.get('marketing_campaign_tracker.trackitem')

            for trackvalues in tracks:
                # create a trackitem for each URL that is replaced
                trackitem.create(cursor, user, trackvalues, context=None)

            html = template.def_body_html
            template.def_body_html, tracks = insert_tracker_in_html(
                html, tracker_base, context['activity_id'])

            for trackvalues in tracks:
                # create a trackitem for each URL that is replaced
                trackitem.create(cursor, user, trackvalues, context=None)

        # END OF CHANGES FROM ORIGINAL METHOD

        result = True
        mailbox_obj = self.pool.get('email_template.mailbox')
        for record_id in record_ids:
            mailbox_id = self._generate_mailbox_item_from_template(
                cursor, user, template, record_id, context)

            mail = mailbox_obj.browse(cursor, user, mailbox_id, context=context)

            if template.report_template or template.attachment_ids:
                self.generate_attach_reports(cursor, user, template,
                                             record_id, mail, context)

            self.pool.get('email_template.mailbox').write(
                cursor, user, mailbox_id, {'folder':'outbox'}, context=context)

            # TODO : manage return value of all the records
            result = self.pool.get('email_template.mailbox').send_this_mail(
                cursor, user, [mailbox_id], context)

        return result

hooked_email_template()


class marketing_campaign_activity(osv.osv):
    # we want to change the marketing campaign activity...
    _inherit = 'marketing.campaign.activity'

    # here we add a new field to the marketing campaign activity
    # the use_tracker boolean will activate the tracking for URLs if is
    # set to True. The default value will be False in order to avoid any
    # problem with existing activities previous to this module installation.
    _columns = {
        'use_tracker': fields.boolean('Use Tracker',
                help='This option activates the usage of the external "Bounced" click tracker'),
        'tracker_base': fields.char("Tracker Base URL", size=200,
                help="The base URL on which the tracker listens"),
        }

    # make sure our new field has a sensible default.
    # see comment above the _columns definition for more
    _defaults = dict(
        use_tracker = False,
        tracker_base = None,
    )

    # make sure an activity which uses a tracker always has a tracker
    # base defined...
    def _check_baseurl(self, cr, uid, ids, context=None):
        """Validates that every activity which has the use_tracker option set
        also has a tracker_base defined
        """
        activities = self.browse(cr, uid, ids, context=context)

        for activity in activities:
            if activity.use_tracker and not activity.tracker_base:
                return False

        return True

    _constraints = [
        (_check_baseurl,
        'If you activate tracking you must give a tracker base',
        ['tracker_base'])
    ]

    # override the _process_wi_email to replace all the URLs in the template
    # body by our tracker URL and store the corresponding keys in database
    def _process_wi_email(self, cr, uid, activity, workitem, context=None):
        if context is None:
            context = {}

        context['use_tracker'] = activity.use_tracker

        if activity.use_tracker:
            # remove any trailing slash that could be present in the base_url
            context['tracker_base'] = activity.tracker_base.rstrip('/')
            context['activity_id'] = activity.id

        return self.pool.get('email.template').generate_mail(cr, uid,
                                            activity.email_template_id.id,
                                            [workitem.res_id], context=context)

marketing_campaign_activity()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

