__author__ = 'faide'

import uuid
import re

simple_text_url_re = re.compile(r'(http://[-a-zA-Z0-9_/.?&=]+)')
html_href_re = re.compile(r'href="([-a-zA-Z0-9_/.?&=:]+)"')

def insert_tracker_in_text(text, tracker_base, activity_id):

    tracks = []

    def repl(match_obj):
        # here we must keep a reference on the uuid in our db for the tracker
        real_url = match_obj.group(1)
        uid = str(uuid.uuid4())
        values = dict(
            trackitem_uuid = uid,
            real_url = real_url,
            campaign_activity_id = activity_id,
            )
        tracks.append(values)
        return "%s/%s" % (tracker_base, uid)

    return simple_text_url_re.sub(repl, text), tracks

def insert_tracker_in_html(html, tracker_base, activity_id):

    tracks = []

    def repl(match_obj):
        # here we must keep a reference on the uuid in our db for the tracker
        real_url = match_obj.group(1)
        uid = str(uuid.uuid4())
        values = dict(
            trackitem_uuid = uid,
            real_url = real_url,
            campaign_activity_id = activity_id,
            )
        tracks.append(values)
        return 'href="%s/%s"' % (tracker_base, uid)

    return html_href_re.sub(repl, html), tracks

if __name__ == '__main__':
    url = "http://some.org/a_3/super-script.php?uid=3&someval=4"
    text = url
    base_html = '<a href="%s"> Some Super Promo </a>'
    html = base_html % url

    print "*"*35
    print "Testing text replacement"
    new_text, values = insert_tracker_in_text(text, 'BASE', '1')

    assert len(values) == 1, "Values should contain only one record"

    value = values[0]
    track_uuid = value.get('trackitem_uuid', None)

    assert track_uuid, "Track uuid should not be None"

    real_url = value.get("real_url", None)
    print "Real URL was: %s" % real_url

    assert real_url == url, "Real URL should have been %s, not %s" % (
                url, real_url)
    assert new_text == '%s/%s' % ('BASE', track_uuid)


    print "*"*35
    print "Testing HTML replacement"
    new_html, values = insert_tracker_in_html(html, 'BASE', '1')

    print new_html
    assert len(values) == 1, "Values should have one record, not %s" % len(values)

    value = values[0]
    track_uuid = value.get('trackitem_uuid', None)

    assert track_uuid, "Track uuid should not be None"

    real_url = value.get("real_url", None)
    print "Real URL was: %s" % real_url

    assert real_url == url, "Real URL should have been %s, not %s" % (
        url, real_url)
    expected_html = base_html % ('BASE/' + track_uuid)
    assert new_html == expected_html
    print "*"*35

